/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef DEFINICE_BAZE_H_INCLUDED
#define DEFINICE_BAZE_H_INCLUDED

#include "porovnej_definice.h"
#include "qstring.h"
#include <set>
#include <vector>
#include "sada_definic.h"
#include "syntaxstrom.h"

/**
 * Zaklad pro ulozeni definice vzorce
 * @author Martin Plicka
 */
class DefiniceBaze {

   public:

      /**
       * cislo radku, na kterem byl vzorec definovan
       */
      int cisloRadku;

      /**
       * seznam parametru funkce
       */
      std::vector<QString> parametry;

      /**
       * Priznak, zda-li se jedna o funkci. Tj. jine chovani a nakladani s cache
       */
      bool _jeFce;

      /**
       * Ukazatel na vrchol stromu vyrazu definujiciho vzorec
       */
      SVyraz * vyraz;

      /**
       * Seznam jinych definic, na kterych tato definice zavisi
       */
      SadaDefinic zavisiNa;

      /**
       * Seznam jinych definic, na kterych tato definice zavisi, majici v ceste podmineny vyraz.
       * (jine nakladani s detekci rekurze).
       */
      SadaDefinic zavisiPresIf;

      /**
       * Konstruktor. Nastavi soucasne cislo radku
       */
      DefiniceBaze();

      /**
       * Pokusi se pridat parametr, automaticky nastavi typ definice na funkci.
       * @param s jmeno parametru
       * @return true, pokud bylo pridani uspesne. Pokud je jmeno duplicitni, vraci false
       */
      bool pridejParametr (const QString & s);

      /**
       * vraci i-ty parametr funkce
       * @param i index
       * @return referenci na i-ty parametr.
       */
      QString & operator[](int i);

      /**
       * Smaze vsechny pozicni parametry definice (tj. funkce)
       */
      void smazParametry();

      /**
       * vraci index parametru
       * @param s nazev hledaneho parametru
       * @return index parametru, pokud neexistuje, vraci -1
       */
      int indexParametru(const QString & s);

      /**
       * Vraci status vzorce, zda-li je funkci, funkce s 0 parametry ma jinou semantiku nez promenna.
       * @return vraci true, pokud je definice funkci.
       */
      virtual bool jeFce();

      /**
       * nastavi priznak, zda se jedna o funkci, kdyz false -> smaze parametry
       * @param b hodnota priznaku
       */
      void jeFce(bool b);

      /**
       * vraci aktualni pocet parametru
       * @return pocet parametru
       */
      virtual int pocetParametru();

      /**
       * zkopiruje nastaveni z jine Definice, resp z LogDefinice
       * @param b reference na kopirovanou definici
       */
      void nastav (const DefiniceBaze& b);

      /**
       * Vygeneruje HTML kod reprezentujici vzorec
       * @return text formatovany HTML
       */
      QString spolecneEcho();

      /**
       * Vymaze seznamy zavislosti
       */
      void smazZavislosti();

      /**
       * smaze z mnozin zavislosti dany prvek, vetsinou se pouziva na odkaz na sama sebe
       * @param d ukazatel na definici, kterou ze zavislosti odebrat.
       */
      void smazZavislost(Definice*d);

      /**
       * Spocte zavislosti
       */
      void zjistiZavislosti();

      /**
       * Ladici funkce, vzpise seznamy zavislosti
       * @return retezec vypisu zavislosti
       */
      QString echoZavislosti();

      /**
       * zkontroluje, zda nektery z identifikatoru, na kterych je definice zavisla, neni dana definice
       * vola se rekurzivne pro vsechny zavislosti,
       * @param d hledana definice
       * @param presIf pokud je true, je zavislost jiz nyni pres If,
       * @throw Vyjimka , pokud je prima rekurze.
       * Pokud je druhy parametr true, znamena to, ze jiz je vnoreno do if, takze prima rekurze nehrozi, hleda se pouze neprima
       * @return true, pokud je zavislost neprima (pres If), jinak false.
       */
      bool zkontrolujRekurzi(Definice *d, bool presIf);
};

#endif // DEFINICE_BAZE_H_INCLUDED
