/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef LOG_H_INCLUDED
#define LOG_H_INCLUDED

#include <list>
#include "definice.h"
#include "syntaxstrom.h"

using std::list;


/**
 * typy vypisu v logu
 */
typedef enum LogTyp {LogTypRadek, LogTypDefinice, LogTypPrikazWrite};

/**
 * Radek logu
 * @author Martin Plicka
 */
struct LogRadek{
   /**
    * Obsah pripadneho chyboveho hlaseni
    */
   QString chyboveHlaseni;

   /**
    * Oznaceni typu kvuli detekci typu objektu
    */
   LogTyp typ;

   /**
    * Konstruktor
    * @param chyba pripadny chybovy text
    */
   LogRadek(const QString & chyba = "");

   /**
    * vypis
    * @return retezec s formatovanym vypisem radku
    */
   virtual QString echo();

   virtual ~LogRadek(){};
};


/**
 * Nerozpoznany vstup
 */
struct LogNerozpoznanyVstup: public LogRadek {
   /**
    * Text nerozpoznaneho vstupu
    */
   QString vstup;

   /**
    * Konstruktor
    * @param vstup text vstupu
    * @param chyba text chyby
    */
   LogNerozpoznanyVstup(const QString &vstup, const QString &chyba);

   virtual QString echo();
};

/**
 * Prikaz
 */
struct LogPrikaz: public LogRadek{
   /**
    * komentar na konci radku
    */
   QString komentar;
   virtual QString echo();
};

/**
 * Prikaz Write (s vystupem)
 */
struct LogPrikazWrite: public LogPrikaz{
   /**
    * vystupni retezec pod prikazem
    */
   QString vystup;

   /**
    * Parametry prikazu
    */
   vector<SVyraz *> parametry;

   /**
    * Konstruktor
    */
   LogPrikazWrite();

   ~LogPrikazWrite();

   /**
    * Prida parametr k prikazu
    * @param p parametr ve forme vyrazu
    */
   void pridejParametr(SVyraz * p);

   /**
    * Vygeneruje vystup, tento vystup se jiz nezmeni
    */
   void vygenerujVystup();

   virtual QString echo();
};

/**
 * Prikaz vypisu definovanych promennych/funkci
 */
struct LogPrikazDef: public LogPrikaz{
   /**
    * vystupni text
    */
   QString vystup;

   LogPrikazDef();

   virtual QString echo();
};

/**
 * Prikaz vypisu zabudovanych konstant/funkci
 */
struct LogPrikazDefInt: public LogPrikaz{

   virtual QString echo();

};

/**
 * Vypis definice promenne, funkce
 */
struct LogDefinice: public LogPrikaz, public DefiniceBaze{
   /**
    * odkaz na definici, kvuli obarvovani
    */
   Definice * definice;

   /**
    * Kontruktor
    * @param d odkaz na definici, je potreba kvuli obarvovani (platnost cache)
    */
   LogDefinice(Definice * d);

   virtual ~LogDefinice();

   virtual QString echo();
};

/**
 * Trida predstavujici cely log programu
 * @author Martin Plicka
 */
class Log {
      /**
       * Seznam radku
       */
      list<LogRadek *> seznam;

   public:
      /**
       * Aktualni cislo radku
       */
      int cisloRadku;

      Log();

      /**
       * Prida radek na konec
       * @param radek vkladany radek
       */
      void pridej (LogRadek * radek);

      /**
       * Smaze cely log
       */
      void smaz();

      /**
       * Vypise cely log
       * @return retezec s vypisem logu
       */
      QString echo();
};


#endif // LOG_H_INCLUDED
