/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "syntaxstrom.h"
#include "strtools.h"
#include "vyjimka.h"
#include <qstring.h>
#include <math.h>
#include "definice.h"

#include "prekladac.h"
#include "prompt.h"


extern Prompt * dVstup;


void SVyraz::zjistiZavislosti(SadaDefinic *a, SadaDefinic *b, bool vIf){
   //nemame co zjistovat
   return;
}


SCislo::SCislo(double h) : SVyraz() {
   hodnota =h;
   typ=Cislo;
}

QString SCislo::echo() {
   return QString("<SPAN CLASS=\"Cislo\">")+Stringify(hodnota,8)+"</SPAN>";
}

double SCislo::eval(ParametryFce * par){
   return hodnota;
}

SRetez::SRetez(QString s): SVyraz() {
   hodnota=s;
   typ=Retez;
}

QString SRetez::echo() {
   return QString("<SPAN CLASS=\"Retez\">\"")+(hodnota)+"\"</SPAN>";
}

double SRetez::eval(ParametryFce * par){
   return 0;
}

/******* UNARNI OPERATORY, NOT, UNMINUS, ZAVORKY***********/

SUnOp::SUnOp (SVyraz * o) {
   op = o;
}

SUnOp::~SUnOp () {
   if (op) delete op;
}

void SUnOp::zjistiZavislosti(SadaDefinic *a, SadaDefinic *b, bool vIf){
   if (op) op->zjistiZavislosti(a,b,vIf);
   return;
}


SZavorky::SZavorky(SVyraz * o): SUnOp(o) {
   typ = Zavorky;
}

QString SZavorky::echo (){
   if (!op) throw VyjimkaText ("Chybi operand");
   return QString("<SPAN CLASS=\"Zavorka\">( </SPAN>")+op->echo()+" <SPAN CLASS=\"Zavorka\">)</SPAN>";
}

double SZavorky::eval(ParametryFce * par){
   return op->eval(par);
}

SNot::SNot(SVyraz * o): SUnOp(o) {
   typ = Not;
}

QString SNot::echo (){
   if (!op) throw VyjimkaText ("Chybi operand");
   return "<SPAN CLASS=\"Not\"> !</SPAN>"+op->echo();
}

double SNot::eval(ParametryFce*par){
   double tmp = op->eval(par);
   return (tmp==0)?1:0;
}

SUnMinus::SUnMinus(SVyraz * o): SUnOp(o) {
   typ = UnMinus;
}

QString SUnMinus::echo (){
   if (!op) throw VyjimkaText ("Chybi operand");
   return "<SPAN CLASS=\"Not\"> -</SPAN>"+op->echo();
}

double SUnMinus::eval(ParametryFce * par){
   return -(op->eval(par));
}

/***************BINARNI OPERATORY*********************/

SBinOp::SBinOp (SVyraz * a, SVyraz * b) {
   opA = a;
   opB = b;
}

SBinOp::~SBinOp () {
   if (opA) delete opA;
   if (opB) delete opB;
}

void SBinOp::zjistiZavislosti(SadaDefinic *a, SadaDefinic *b, bool vIf){
   if (opA) opA->zjistiZavislosti(a,b,vIf);
   if (opB) opB->zjistiZavislosti(a,b,vIf);
   return;
}



SPlus::SPlus(SVyraz * a, SVyraz *b):SBinOp(a,b){
   typ = Plus;
}

QString SPlus::echo() {
   if (!opA || !opB) throw VyjimkaText ("Chybi operandy");
   return opA->echo()+" <SPAN CLASS=\"Plus\">+</SPAN> "+opB->echo();
}

double SPlus::eval(ParametryFce * par){
   return (opA->eval(par)) + (opB->eval(par));
}

SMinus::SMinus(SVyraz * a, SVyraz *b):SBinOp(a,b){
   typ = Minus;
}

QString SMinus::echo() {
   if (!opA || !opB) throw VyjimkaText ("Chybi operandy");
   return opA->echo()+" <SPAN CLASS=\"Minus\">-</SPAN> "+opB->echo();
}

double SMinus::eval(ParametryFce * par){
   return (opA->eval(par)) - (opB->eval(par));
}

SKrat::SKrat(SVyraz * a, SVyraz *b):SBinOp(a,b){
   typ = Krat;
}

QString SKrat::echo() {
   if (!opA || !opB) throw VyjimkaText ("Chybi operandy");
   return opA->echo()+" <SPAN CLASS=\"Krat\">*</SPAN> "+opB->echo();
}

double SKrat::eval(ParametryFce * par){
   double tmp = opA->eval(par);

   // pravy operand nevyhodnocujeme, pokud to neni treba
   if (tmp==0) return 0;
   else return tmp * (opB->eval(par));
}

SDeleno::SDeleno(SVyraz * a, SVyraz *b):SBinOp(a,b){
   typ = Deleno;
}

QString SDeleno::echo() {
   if (!opA || !opB) throw VyjimkaText ("Chybi operandy");
   return opA->echo()+QString(" <SPAN CLASS=\"Deleno\">/</SPAN> ")+opB->echo();
}

double SDeleno::eval(ParametryFce * par){
   // zkontrolujeme deleni nulou
   double delitel = opB->eval(par);
   if (delitel==0) throw VyjimkaText("Deleni nulou");
   return (opA->eval(par)) / (delitel);
}


SMocnina::SMocnina(SVyraz * a, SVyraz * b):SBinOp(a,b){
   typ = Mocnina;
}

QString SMocnina::echo() {
   if (!opA || !opB) throw VyjimkaText ("Chybi operandy");
   return opA->echo()+QString(" <SPAN CLASS=\"Mocnina\">^</SPAN> ")+opB->echo();
}

double SMocnina::eval(ParametryFce * par){
   return pow((opA->eval(par)),(opB->eval(par)));
}


SAnd::SAnd(SVyraz * a, SVyraz * b):SBinOp(a,b){
   typ = And;
}

QString SAnd::echo() {
   if (!opA || !opB) throw VyjimkaText ("Chybi operandy");
   return opA->echo()+QString(" <SPAN CLASS=\"And\">&amp;</SPAN> ")+opB->echo();
}

double SAnd::eval(ParametryFce * par){

   //kdyz je prvni parametr false(0), neni treba dal vyhodnocovat
   double tmp = opA->eval(par);
   if (tmp==0) return 0;
   return (opB->eval(par) !=0)?1:0;
}


SOr::SOr(SVyraz * a, SVyraz * b):SBinOp(a,b){
   typ = Or;
}

QString SOr::echo() {
   if (!opA || !opB) throw VyjimkaText ("Chybi operandy");
   return opA->echo()+QString(" <SPAN CLASS=\"Or\">|</SPAN> ")+opB->echo();
}

double SOr::eval(ParametryFce * par){

   //pokud je true, netreba dal vyhodnocovat
   double tmp = opA->eval(par);
   if (tmp!=0) return 1;
   return (opB->eval(par) !=0)?1:0;
}


SVetsi::SVetsi(SVyraz * a, SVyraz * b):SBinOp(a,b){
   typ = Vetsi;
}

QString SVetsi::echo() {
   if (!opA || !opB) throw VyjimkaText ("Chybi operandy");
   return opA->echo()+QString(" <SPAN CLASS=\"Vetsi\">&gt;</SPAN> ")+opB->echo();
}

double SVetsi::eval(ParametryFce * par){
   if ( (opA->eval(par)) > (opB->eval(par))) return 1;
   else return 0;
}


SMensi::SMensi(SVyraz * a, SVyraz * b):SBinOp(a,b){
   typ = Mensi;
}

QString SMensi::echo() {
   if (!opA || !opB) throw VyjimkaText ("Chybi operandy");
   return opA->echo()+QString(" <SPAN CLASS=\"Mensi\">&lt;</SPAN> ")+opB->echo();
}

double SMensi::eval(ParametryFce * par){
   if ( (opA->eval(par)) < (opB->eval(par))) return 1;
   else return 0;
}

SVetsiRovno::SVetsiRovno(SVyraz * a, SVyraz * b):SBinOp(a,b){
   typ = VetsiRovno;
}

QString SVetsiRovno::echo() {
   if (!opA || !opB) throw VyjimkaText ("Chybi operandy");
   return opA->echo()+QString(" <SPAN CLASS=\"VetsiRovno\">&ge;</SPAN> ")+opB->echo();
}

double SVetsiRovno::eval(ParametryFce * par){
   if ( (opA->eval(par)) >= (opB->eval(par))) return 1;
   else return 0;
}

SMensiRovno::SMensiRovno(SVyraz * a, SVyraz * b):SBinOp(a,b){
   typ = MensiRovno;
}

QString SMensiRovno::echo() {
   if (!opA || !opB) throw VyjimkaText ("Chybi operandy");
   return opA->echo()+QString(" <SPAN CLASS=\"MensiRovno\">&le;</SPAN> ")+opB->echo();
}

double SMensiRovno::eval(ParametryFce * par){
   if ( (opA->eval(par)) <= (opB->eval(par))) return 1;
   else return 0;
}

SRovno::SRovno(SVyraz * a, SVyraz * b):SBinOp(a,b){
   typ = Rovno;
}

QString SRovno::echo() {
   if (!opA || !opB) throw VyjimkaText ("Chybi operandy");
   return opA->echo()+QString(" <SPAN CLASS=\"Rovno\">==</SPAN> ")+opB->echo();
}

double SRovno::eval(ParametryFce * par){
   if ( (opA->eval(par)) == (opB->eval(par))) return 1;
   else return 0;
}

SNerovno::SNerovno(SVyraz * a, SVyraz * b):SBinOp(a,b){
   typ = Nerovno;
}

QString SNerovno::echo() {
   if (!opA || !opB) throw VyjimkaText ("Chybi operandy");
   return opA->echo()+QString(" <SPAN CLASS=\"Nerovno\">&ne;</SPAN> ")+opB->echo();
}

double SNerovno::eval(ParametryFce * par){
   if ( (opA->eval(par)) != (opB->eval(par))) return 1;
   else return 0;
}



/***************FUNKCE nebo PROMENNA (funkce bez parametru) *****************************/

SIdentifikator::SIdentifikator (Definice * d) {
   typ = Identifikator;
   definice = d;
   _jeFce=false;
}

SIdentifikator::~SIdentifikator () {
   for (vector<SVyraz*>::iterator tmp= parametry.begin(); tmp!=parametry.end(); tmp++) delete *tmp;
}

void SIdentifikator::pridejParametr (SVyraz * s){
   _jeFce = true;
   parametry.push_back(s);
}

void SIdentifikator::smazParametry() {
   parametry.clear();
}

bool SIdentifikator::jeFce(){
   return _jeFce;
}

bool SIdentifikator::jeFce(bool b) {
   _jeFce = b;
   if (!b) smazParametry();
   return b;
}

int SIdentifikator::pocetParametru() {
   return parametry.size();
}

QString SIdentifikator::echo() {
   QString str=QString("<SPAN CLASS=\"%1\">").arg((jeFce())?("IdentifikatorFce"):("Identifikator"));
   if (definice->cachePlatna && !(definice->jeFce())) str+=QString("<SPAN CLASS=\"IdentifikatorPlatny\">");
   str+= definice->jmeno;
   if (definice->cachePlatna && !(definice->jeFce())) str+="</SPAN>";
   str+="</SPAN>";
   if (!jeFce()) return str;
   str+=QString("<SPAN CLASS=\"ZavorkaFce\">(</SPAN> ");
   bool a = true;

   for (vector<SVyraz*>::iterator tmp= parametry.begin(); tmp!=parametry.end(); tmp++) {
      if (a) a=false;
      else str+=QString(" <SPAN CLASS=\"ZavorkaFce\">,</SPAN> ");
      str+= (*tmp)->echo();
   }
   str+=QString(" <SPAN CLASS=\"ZavorkaFce\">)</SPAN>");
   return str;
}

void SIdentifikator::zjistiZavislosti(SadaDefinic *a, SadaDefinic *b, bool vIf){

   if (!definice->zabudovana) ((!vIf)?a:b)->insert(definice); //pridej svoji definici,
   // pouze pokud neni funkce zabudovana, zavislosti na zabudovanych funkcich nepotrebujeme

   //a posli to dal vsem parametrum
   for (vector<SVyraz*>::iterator tmp= parametry.begin(); tmp!=parametry.end(); tmp++) {
      (*tmp)->zjistiZavislosti(a,b,vIf);
   }
   return;
}

double SIdentifikator::eval(ParametryFce * par){
   // zkontrolovat
   if (!(definice->jeDefinovana())) throw VyjimkaText("Proměnná/funkce <B>\""+definice->jmeno+"\"</B> není definována");
   if (jeFce() && !(definice->jeFce())) throw VyjimkaText("Proměnná <B>\""+definice->jmeno+"\"</B> je použita jako funkce");
   if (!jeFce() && (definice->jeFce())) throw VyjimkaText("Funkce <B>\""+definice->jmeno+"\"</B> je použita jako proměnná ");
   if (pocetParametru() > (definice->pocetParametru())) throw VyjimkaText("Funkce <B>\""+definice->jmeno+"\"</B> je volána s více parametry, než je definováno");

   //inkrementuj zanoreni
   Prekladac::vnoreni++;

   //spocti parametry funkce
   ParametryFce novePar;
   for (vector<SVyraz*>::iterator tmp= parametry.begin(); tmp!=parametry.end(); tmp++)
      novePar.pridej((*tmp)->eval (par));

   double tmp =  definice->eval(&novePar);

   //dekrementuj zanoreni
   Prekladac::vnoreni--;

   return tmp;
}


/*********LOKALNI PROMENNA ******************************/

SLokalniPromenna::SLokalniPromenna(int i, DefiniceBaze * p){
   typ = LokalniPromenna;
   index = i;
   ptr = p;
}

QString SLokalniPromenna::echo(){
   return QString("<SPAN CLASS=\"IdentifikatorParametr\">")+((ptr)?(*ptr)[index]:(QString("")))+"</SPAN>";
}

double SLokalniPromenna::eval(ParametryFce * par){
   return (*par)[index];
}

/******* PODMINENY VYRAZ **************************/

SIf::SIf(SVyraz * p, SVyraz *a, SVyraz *n){
   typ = If;
   podminka = p;
   kdyzPravda = a;
   kdyzNepravda = n;
}

QString SIf::echo() {
   if (!podminka || !kdyzPravda || !kdyzNepravda) throw VyjimkaText("Chyba programu, IF neni zapsan spravne");
   QString str=QString("<SPAN CLASS=\"If\">if</SPAN>");
   str+=QString("<SPAN CLASS=\"ZavorkaIf\">(</SPAN> ");
   str+=podminka->echo();
   str+=QString(" <SPAN CLASS=\"ZavorkaIf\">,</SPAN> ");
   str+=kdyzPravda->echo();
   str+=QString(" <SPAN CLASS=\"ZavorkaIf\">,</SPAN> ");
   str+=kdyzNepravda->echo();
   str+=QString(" <SPAN CLASS=\"ZavorkaIf\">)</SPAN>");
   return str;
}

SIf::~SIf (){
   if (podminka) delete podminka;
   if (kdyzPravda) delete kdyzPravda;
   if (kdyzNepravda) delete kdyzNepravda;
}

void SIf::zjistiZavislosti(SadaDefinic *a, SadaDefinic *b, bool vIf){
   if (podminka) podminka->zjistiZavislosti(a,b,vIf);

   //zbylym dvema dame natvrdo vIf=true
   if (kdyzPravda) kdyzPravda->zjistiZavislosti(a,b,true);
   if (kdyzNepravda) kdyzNepravda->zjistiZavislosti(a,b,true);
}

double SIf::eval(ParametryFce*par){
   double tmp = podminka->eval(par);
   if (tmp!=0) return kdyzPravda->eval(par);
   else return kdyzNepravda->eval(par);
}

/************** READ ********************/
SRead::SRead(SRetez * r){
   text = r;
   typ=Read;
}

SRead::~SRead(){
   if (text) delete text;
}

QString SRead::echo(){
   QString str=QString("<SPAN CLASS=\"IdentifikatorFce\">read</SPAN>");
   str+=QString("<SPAN CLASS=\"ZavorkaFce\">(</SPAN> ");
   str+=(text)?(text->echo()):("");
   str+=QString(" <SPAN CLASS=\"ZavorkaFce\">)</SPAN>");
   return str;
}
double SRead::eval(ParametryFce*par){
   return dVstup->cti(text->hodnota);
}
