/*
 *   MiniMath, mathematic operations interpreter.
 *   Copyright 2006,2007,2008 Martin Plicka
 *
 *   This file is part of MiniMath.
 *
 *   MiniMath is free software: you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *   MiniMath is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with MiniMath.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __SYNTAXSTROM_H_
#define __SYNTAXSTROM_H_

#include <vector>
#include <set>
#include "qstring.h"
#include "parametry_fce.h"
#include "sada_definic.h"

class DefiniceBaze;

using std::vector;
using std::set;

/**
 * Typ prvku syntaktickeho stromu
 */
typedef enum STypVyrazu {
   Identifikator, Cislo, Retez,
   Plus, Minus, Krat, Deleno, Mocnina,
   And, Or, Not,
   UnMinus,
   Zavorky,
   Vetsi, Mensi, VetsiRovno, MensiRovno, Rovno, Nerovno,
   LokalniPromenna, If, Read,
};

/**
 * Zakladni trida pro typy prvku stromu
 */
struct SVyraz {
   /**
    * Typ vyrazu.
    * Slouzi pro lepsi identifikaci typu prvku
    */
   STypVyrazu typ;

   virtual ~SVyraz(){};

   /**
    * Vypise prislusny podstrom vnitrni formy
    * @return retezec s vypisem
    */
   virtual QString echo ()=0;

   /**
    * Vyhodnoti prislusny podstrom
    * @param parametry vypoctene parametry funkce, ktera obsahuje dany podstrom.
    * Je nutno je predavat nize pro potreby SLokalniPromenna.
    * @return vypoctena hodnota
    */
   virtual double eval(ParametryFce * parametry)=0;

   /**
    * Prochazi smer smerem dolu a do predanych kolekci vklada odkazy na definice
    * @param a kolekce primych definic
    * @param b kolekce neprimych definic
    * @param vIf priznak, zda-li je aktualni pozice dosazena pres If.
    */
   virtual void zjistiZavislosti (SadaDefinic *a, SadaDefinic *b, bool vIf);
};

/***************** KONSTANTY ******************************/

/**
 * Cislo
 */
struct SCislo: public SVyraz {
   /**
    * Hodnota cisla
    */
   double hodnota;

   /**
    * Konstruktor
    * @param h hodnota cisla
    */
   SCislo(double h=0);

   virtual double eval(ParametryFce *);
   virtual QString echo();
};

/**
 * Retezec
 */
struct SRetez: public SVyraz { //retezec
   /**
    * Hodnota retezce
    */
   QString hodnota;

   /**
    * Konstruktor
    * @param s hodnota retezce
    */
   SRetez(QString s="");

   virtual double eval(ParametryFce *);
   virtual QString echo();
};


/******* UNARNI OPERATORY, NOT, UNMINUS, ZAVORKY***********/

/**
 * Abstraktni trida pro unarni operatory
 */
struct SUnOp: public SVyraz{ //abstract pro unarni operatory
   /**
    * Operand
    */
   SVyraz * op;

   /**
    * Konstruktor
    * @param o operand
    */
   SUnOp (SVyraz * o=NULL);

   virtual ~SUnOp ();
   virtual void zjistiZavislosti(SadaDefinic *a, SadaDefinic *b, bool vIf);
   virtual double eval(ParametryFce *)=0;
};

/**
 * Zavorky, jsou vlastne unarnim operatorem
 */
struct SZavorky: public SUnOp{ //zavorky jsou take unarni operator
   /**
    * Konstruktor
    * @param o operand
    */
   SZavorky(SVyraz * o=NULL);

   virtual QString echo ();
   virtual double eval(ParametryFce *);
};

/**
 * Unarni NOT
 */
struct SNot: public SUnOp{
   /**
    * Konstruktor
    * @param o operand
    */
   SNot(SVyraz * o=NULL);

   virtual QString echo ();
   virtual double eval(ParametryFce *);
};

/**
 * Unarni minus
 */
struct SUnMinus: public SUnOp{
   /**
    * Konstruktor
    * @param o operand
    */
   SUnMinus(SVyraz * o=NULL);

   virtual QString echo ();
   virtual double eval(ParametryFce *);
};

/***************BINARNI OPERATORY*********************/

/**
 * Abstraktni trida binarniho operatoru
 */
struct SBinOp: public SVyraz{ //abstract pro binarni operatory

   /**
    * Operand A
    */
   SVyraz *opA;

   /**
    * Operand B
    */
   SVyraz *opB; // dva operandy

   /**
    * Konstruktor
    * @param a operand A
    * @param b operand B
    */
   SBinOp (SVyraz * a=NULL, SVyraz * b=NULL);

   virtual ~SBinOp ();
   virtual QString echo()=0;
   virtual void zjistiZavislosti(SadaDefinic *a, SadaDefinic *b, bool vIf);
   virtual double eval(ParametryFce *)=0;
};

/**
 * Binarni plus
 */
struct SPlus: public SBinOp {

   /**
    * Konstruktor
    * @param a operand A
    * @param b operand B
    */
   SPlus(SVyraz * a=NULL, SVyraz *b=NULL);

   virtual QString echo();
   virtual double eval(ParametryFce *);
};

/**
 * Binarni minus
 */
struct SMinus: public SBinOp {
   /**
    * Konstruktor
    * @param a operand A
    * @param b operand B
    */
   SMinus(SVyraz * a=NULL, SVyraz *b=NULL);

   virtual QString echo();
   virtual double eval(ParametryFce *);
};

/**
 * Binarni krat
 */
struct SKrat: public SBinOp {
   /**
    * Konstruktor
    * @param a operand A
    * @param b operand B
    */
   SKrat(SVyraz * a=NULL, SVyraz *b=NULL);

   virtual QString echo();
   virtual double eval(ParametryFce *);
};

/**
 * Deleno
 */
struct SDeleno: public SBinOp {

   /**
    * Konstruktor
    * @param a operand A
    * @param b operand B
    */
   SDeleno(SVyraz * a=NULL, SVyraz *b=NULL);

   virtual QString echo();
   virtual double eval(ParametryFce *);
};

/**
 * Obecna mocnina
 */
struct SMocnina: public SBinOp {

   /**
    * Konstruktor
    * @param a operand A
    * @param b operand B
    */
   SMocnina(SVyraz * a=NULL, SVyraz * b= NULL);

   virtual QString echo();
   virtual double eval(ParametryFce *);
};

/**
 * AND
 */
struct SAnd: public SBinOp {

   /**
    * Konstruktor
    * @param a operand A
    * @param b operand B
    */
   SAnd(SVyraz * a=NULL, SVyraz * b= NULL);

   virtual QString echo();
   virtual double eval(ParametryFce *);
};

/**
 * OR
 */
struct SOr: public SBinOp {

   /**
    * Konstruktor
    * @param a operand A
    * @param b operand B
    */
   SOr(SVyraz * a=NULL, SVyraz * b= NULL);

   virtual QString echo();
   virtual double eval(ParametryFce *);
};

/**
 * Relacni operator > (je vetsi)
 */
struct SVetsi: public SBinOp {

   /**
    * Konstruktor
    * @param a operand A
    * @param b operand B
    */
   SVetsi(SVyraz * a=NULL, SVyraz * b= NULL);

   virtual QString echo();
   virtual double eval(ParametryFce *);
};

/**
 * Relacni operator < (je mensi)
 */
struct SMensi: public SBinOp {

   /**
    * Konstruktor
    * @param a operand A
    * @param b operand B
    */
   SMensi(SVyraz * a=NULL, SVyraz * b= NULL);

   virtual QString echo();
   virtual double eval(ParametryFce *);
};

/**
 * Relacni operator >= (vetsi nebo rovno)
 */
struct SVetsiRovno: public SBinOp {

   /**
    * Konstruktor
    * @param a operand A
    * @param b operand B
    */
   SVetsiRovno(SVyraz * a=NULL, SVyraz * b= NULL);

   virtual QString echo();
   virtual double eval(ParametryFce *);
};

/**
 * Relacni operator <= (mensi nebo rovno)
 */
struct SMensiRovno: public SBinOp {

   /**
    * Konstruktor
    * @param a operand A
    * @param b operand B
    */
   SMensiRovno(SVyraz * a=NULL, SVyraz * b= NULL);

   virtual QString echo();
   virtual double eval(ParametryFce *);
};

/**
 * Relacni operator = (je rovno)
 */
struct SRovno: public SBinOp {

   /**
    * Konstruktor
    * @param a operand A
    * @param b operand B
    */
   SRovno(SVyraz * a=NULL, SVyraz * b= NULL);

   virtual QString echo();
   virtual double eval(ParametryFce *);
};

/**
 * Relacni operator <> (nerovna se)
 */
struct SNerovno: public SBinOp {

   /**
    * Konstruktor
    * @param a operand A
    * @param b operand B
    */
   SNerovno(SVyraz * a=NULL, SVyraz * b= NULL);

   virtual QString echo();
   virtual double eval(ParametryFce *);
};

/***************FUNKCE nebo PROMENNA (funkce bez parametru) *****************************/

/**
 * Identifikator - funkce nebo promenna
 */
struct SIdentifikator: public SVyraz{
   /**
    * Seznam parametru funkce
    */
   vector<SVyraz*> parametry; //seznam parametru funkce

   /**
    * Priznak, zda se jedna o funkci nebo promennou
    */
   bool _jeFce;

   /**
    * Odkaz na definici funkce
    */
   Definice * definice;


   /**
    * Konstruktor
    * @param def odkaz na definici
    */
   SIdentifikator (Definice * def);

   virtual ~SIdentifikator ();

   /**
    * Prida parametr do funkce (pri cteni vstupu).
    * @param s vrchol stromu podvyrazu
    */
   void pridejParametr (SVyraz * s);

   /**
    * Vymaz vsechny parametry funkce
    */
   void smazParametry();

   /**
    * Otestuje, zda se jedna o funkci
    * @return true, pokud se jedna o funkci
    */
   bool jeFce();

   /**
    * Nastavi priznak, zdali se jedna o funkci
    * @param b hodnota priznaku
    * @return hodnota priznaku
    */
   bool jeFce(bool b);

   /**
    * Zjisti pocet aktualnich parametru funkce
    * @return pocet parametru funkce
    */
   int pocetParametru();

   virtual QString echo();
   virtual void zjistiZavislosti(SadaDefinic *a, SadaDefinic *b, bool vIf);
   virtual double eval(ParametryFce * par);
};

/********************LOKALNI PROMENNA Z PARAMETRU FUNKCE *****************************************/

/**
 * Lokalni promenna - uvnitr funkce.
 */
struct SLokalniPromenna: public SVyraz {
   /**
    * Vypocteny index v poli parametru definice funkce
    */
   int index;

   /**
    * Odkaz na definici funkce, ktera drzi nazev lokalni promenne
    */
   DefiniceBaze * ptr;

   /**
    * Kontruktor
    * @param i index v poli parametru v definici funkce
    * @param p odkaz na definici nadrazene funkce
    */
   SLokalniPromenna(int i, DefiniceBaze * p);

   virtual QString echo ();
   virtual double eval(ParametryFce * par);
};


/**
 * Podmineny vyraz if (cond, iftrue, iffalse)
 */
struct SIf: public SVyraz {
   /**
    * Podminka vyrazu
    */
   SVyraz * podminka;

   /**
    * Podstrom, ktery se vyhodnoti v pripade pravdive podminky
    */
   SVyraz * kdyzPravda;

   /**
    * Podstrom, ktery se vyhodnoti v pripade NEpravdive podminky
    */
   SVyraz * kdyzNepravda;

   /**
    * Konstruktor
    * @param p podminka
    * @param a hodnota v pripade podminka = true
    * @param n hodnota v pripade podminka = false
    */
   SIf (SVyraz * p=NULL, SVyraz *a=NULL, SVyraz *n=NULL);

   virtual ~SIf ();
   virtual QString echo ();
   virtual void zjistiZavislosti(SadaDefinic *a, SadaDefinic *b, bool vIf);
   virtual double eval(ParametryFce * par);
};

/**
 * Prikaz read(), v dobe vyhodnoceni vyzve uzivatele ke vstupu hodnoty
 */
struct SRead: public SVyraz {
   /**
    * Retezec s vyzvou ke vstupu
    */
   SRetez * text;

   /**
    * Kontruktor
    * @param r retezec s vyzvou ke vstupu
    */
   SRead(SRetez * r=NULL);

   virtual ~SRead();
   virtual QString echo();
   virtual double eval(ParametryFce * par );
};
#endif
